/*
 * Copyright (C) by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file ExceptionCallstackSize.h
 *    Control size of callstacks stored with exceptions
 *
 * @author Christof Schröter
 * @date   2022/03/22
 */

#ifndef _MIRA_EXCEPTION_CALLSTACK_SIZE_H_
#define _MIRA_EXCEPTION_CALLSTACK_SIZE_H_

#include <utils/Singleton.h>

namespace mira {

///////////////////////////////////////////////////////////////////////////////

/**
 * ExceptionCallStackSize is a singleton that provides a mechanism to control
 * the depth of the callstack that is stored with exceptions (e.g. for logging).
 */
class ExceptionCallStackSize : public Singleton<ExceptionCallStackSize>
{
	friend class singleton::CreateUsingNew<ExceptionCallStackSize>;

protected:
	ExceptionCallStackSize() : mMaxSize(MIRA_MAX_CALLSTACK_SIZE) {}

	static const unsigned int MIRA_MAX_CALLSTACK_SIZE = 16;

public:
	/// Set callstack max size
	static void set(unsigned int size) { instance().mMaxSize = size; }

	/**
	 * Reset callstack start level and max size to initial default value
	 * (\ref MIRA_MAX_CALLSTACK_SIZE).
	 */
	static void reset() { instance().mMaxSize = MIRA_MAX_CALLSTACK_SIZE; }

	/// Query callstack max size
	static unsigned int get() { return instance().mMaxSize; }

 private:
	unsigned int mMaxSize;
};

///////////////////////////////////////////////////////////////////////////////

}

#endif
